window.Gleam.initSearch([{"doc":"outcome","title":"outcome","content":"# outcome\n\n[![Package Version](https://img.shields.io/hexpm/v/outcome)](https://hex.pm/packages/outcome)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/outcome/)\n\nAn Error Handling library for Gleam. Inspired by\n&lt;https://github.com/lpil/snag&gt; and &lt;https://effect.website/&gt;.\n\nOutcomes gives you:\n\n- An error stack that makes it easier to track the path of an error.\n- A way to distinguish between unexpected errors (Defect) and expected errors (Failure).\n\n```sh\ngleam add outcome\n```\n\n```gleam\nimport outcome.{type Outcome}\n\nfn using(email) {\n  case signup(email) {\n    Error(stack) -&gt; io.print(outcome.pretty_print(stack))\n    Ok(user) -&gt; {\n      //...\n    }\n  }\n}\n\nfn signup(email: String) -&gt; Outcome(User) {\n  use valid_email &lt;- result.try(\n    validate_email(email)\n    |&gt; outcome.add_context(&quot;In signup&quot;)\n  )\n\n  create_user(valid_email)\n}\n\n// An expected error should be marked as a failure\nfn validate_email(email: String) -&gt; Outcome(String) {\n  Error(&quot;Invalid email&quot;)\n    |&gt; outcome.into_failure\n}\n\n// An unexpected error should be marked as a defect\nfn create_user() -&gt; Outcome(User) {\n  Error(&quot;Some SQL error&quot;)\n  |&gt; outcome.into_defect\n  |&gt; outcome.add_context(&quot;In create_user&quot;)\n}\n```\n\n```gleam\nusing(&quot;invalid&quot;)\n\nFailure: Invalid email\n\nstack:\n  Context: In signup\n  Failure: Invalid email\n```\n\n```gleam\nusing(&quot;sam@sample.com&quot;)\n\nDefect: Some SQL error\n\nstack:\n  Context: In signup\n  Context: In create_user\n  Defect: Some SQL error\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/outcome&gt;.\n","url":"index.html"},{"doc":"outcome","title":"Outcome","content":"pub type Outcome(t) =\n  Result(t, Stack)\n Alias to Result with Stack as error type.\n\nSynonyms:\noutcome.Outcome\noutcome Outcome","url":"outcome.html#Outcome"},{"doc":"outcome","title":"Problem","content":"pub type Problem {\n  Context(String)\n  Defect(String)\n  Failure(String)\n}\n An application error is either a Defect or a Failure.\n A Defect is an unexpected application error, which shouldn&#39;t be shown to the user.\n A Failure is an expected error.\n Context is just information about the place in the application to build a stack trace.\nContext(String)\n\n\nDefect(String)\n\n\nFailure(String)\n\n\nSynonyms:\noutcome.Problem\noutcome Problem","url":"outcome.html#Problem"},{"doc":"outcome","title":"Stack","content":"pub type Stack {\n  Stack(problem: Problem, problems: List(Problem))\n}\n A stack of problems and context.\nStack(problem: Problem, problems: List(Problem))\n\n\nSynonyms:\noutcome.Stack\noutcome Stack","url":"outcome.html#Stack"},{"doc":"outcome","title":"add_context","content":"pub fn add_context(\n  outcome outcome: Result(a, Stack),\n  context context: String,\n) -&gt; Result(a, Stack)\n Add context to an Outcome\nSynonyms:\noutcome.add_context\noutcome add_context","url":"outcome.html#add_context"},{"doc":"outcome","title":"add_context_to_stack","content":"pub fn add_context_to_stack(stack: Stack, value: String) -&gt; Stack\n\nSynonyms:\noutcome.add_context_to_stack\noutcome add_context_to_stack","url":"outcome.html#add_context_to_stack"},{"doc":"outcome","title":"add_defect_to_stack","content":"pub fn add_defect_to_stack(stack: Stack, value: String) -&gt; Stack\n\nSynonyms:\noutcome.add_defect_to_stack\noutcome add_defect_to_stack","url":"outcome.html#add_defect_to_stack"},{"doc":"outcome","title":"add_failure_to_stack","content":"pub fn add_failure_to_stack(stack: Stack, value: String) -&gt; Stack\n\nSynonyms:\noutcome.add_failure_to_stack\noutcome add_failure_to_stack","url":"outcome.html#add_failure_to_stack"},{"doc":"outcome","title":"add_to_stack","content":"pub fn add_to_stack(stack: Stack, new_problem: Problem) -&gt; Stack\n Add a Problem to the top of a Stack\nSynonyms:\noutcome.add_to_stack\noutcome add_to_stack","url":"outcome.html#add_to_stack"},{"doc":"outcome","title":"as_defect","content":"pub fn as_defect(\n  result: Result(a, Nil),\n  e: String,\n) -&gt; Result(a, Stack)\n Replaces an Problem with a wrapped Defect\nSynonyms:\noutcome.as_defect\noutcome as_defect","url":"outcome.html#as_defect"},{"doc":"outcome","title":"as_failure","content":"pub fn as_failure(\n  result: Result(a, Nil),\n  e: String,\n) -&gt; Result(a, Stack)\n Replaces an Problem with a wrapped Failure\nSynonyms:\noutcome.as_failure\noutcome as_failure","url":"outcome.html#as_failure"},{"doc":"outcome","title":"defect","content":"pub fn defect(value: String) -&gt; Problem\n Wrap a String into a Defect\nSynonyms:\noutcome.defect\noutcome defect","url":"outcome.html#defect"},{"doc":"outcome","title":"error_with_defect","content":"pub fn error_with_defect(defect: String) -&gt; Result(a, Stack)\n Create a Defect wrapped in an Problem\nSynonyms:\noutcome.error_with_defect\noutcome error_with_defect","url":"outcome.html#error_with_defect"},{"doc":"outcome","title":"error_with_failure","content":"pub fn error_with_failure(failure: String) -&gt; Result(a, Stack)\n Create Failure wrapped in an Problem\nSynonyms:\noutcome.error_with_failure\noutcome error_with_failure","url":"outcome.html#error_with_failure"},{"doc":"outcome","title":"failure","content":"pub fn failure(value: String) -&gt; Problem\n Wrap a String into a Failure\nSynonyms:\noutcome.failure\noutcome failure","url":"outcome.html#failure"},{"doc":"outcome","title":"into_defect","content":"pub fn into_defect(result: Result(a, String)) -&gt; Result(a, Stack)\n Convert an Problem into a wrapped Defect\nSynonyms:\noutcome.into_defect\noutcome into_defect","url":"outcome.html#into_defect"},{"doc":"outcome","title":"into_failure","content":"pub fn into_failure(\n  result: Result(a, String),\n) -&gt; Result(a, Stack)\n Convert an Problem into a wrapped Failure\nSynonyms:\noutcome.into_failure\noutcome into_failure","url":"outcome.html#into_failure"},{"doc":"outcome","title":"map_into_defect","content":"pub fn map_into_defect(\n  result: Result(a, b),\n  mapper: fn(b) -&gt; String,\n) -&gt; Result(a, Stack)\n Convert an Problem into a wrapped Defect, by using a mapping function\nSynonyms:\noutcome.map_into_defect\noutcome map_into_defect","url":"outcome.html#map_into_defect"},{"doc":"outcome","title":"new_stack","content":"pub fn new_stack(error: Problem) -&gt; Stack\n Create a new Stack with the given Problem\nSynonyms:\noutcome.new_stack\noutcome new_stack","url":"outcome.html#new_stack"},{"doc":"outcome","title":"new_stack_with_defect","content":"pub fn new_stack_with_defect(failure: String) -&gt; Stack\n Create a Defect wrapped in an Stack\nSynonyms:\noutcome.new_stack_with_defect\noutcome new_stack_with_defect","url":"outcome.html#new_stack_with_defect"},{"doc":"outcome","title":"new_stack_with_failure","content":"pub fn new_stack_with_failure(failure: String) -&gt; Stack\n Create a Failure wrapped in an Stack\nSynonyms:\noutcome.new_stack_with_failure\noutcome new_stack_with_failure","url":"outcome.html#new_stack_with_failure"},{"doc":"outcome","title":"pretty_print","content":"pub fn pretty_print(stack: Stack) -&gt; String\n\nSynonyms:\noutcome.pretty_print\noutcome pretty_print","url":"outcome.html#pretty_print"},{"doc":"outcome","title":"stack_to_lines","content":"pub fn stack_to_lines(stack: Stack) -&gt; List(String)\n\nSynonyms:\noutcome.stack_to_lines\noutcome stack_to_lines","url":"outcome.html#stack_to_lines"},{"doc":"outcome","title":"stack_to_problems","content":"pub fn stack_to_problems(stack: Stack) -&gt; List(Problem)\n Get a list of problems for a Stack\nSynonyms:\noutcome.stack_to_problems\noutcome stack_to_problems","url":"outcome.html#stack_to_problems"},{"doc":"outcome","title":"unwrap_failure","content":"pub fn unwrap_failure(stack: Stack, default: String) -&gt; String\n Get the failure at the top of the stack.\n If the top is not a failure, then return the given default.\n Use this for showing an error message to users.\nSynonyms:\noutcome.unwrap_failure\noutcome unwrap_failure","url":"outcome.html#unwrap_failure"},{"doc":"outcome","title":"outcome","content":"","url":"outcome.html"}]);